@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2000-2010 Bastian Kleineidam
+# Copyright (C) 2000-2014 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,12 +17,10 @@
 """
 Handle a queue of URLs to check.
 """
-from __future__ import with_statement
 import threading
 import collections
 from time import time as _time
 from .. import log, LOG_CACHE
-from ..containers import LFUCache
 
 
 class Timeout (StandardError):
@@ -34,11 +32,13 @@ class Empty (StandardError):
     pass
 
 
+NUM_PUTS_CLEANUP = 10000
+
 class UrlQueue (object):
     """A queue supporting several consumer tasks. The task_done() idea is
     from the Python 2.5 implementation of Queue.Queue()."""
 
-    def __init__ (self):
+    def __init__ (self, max_allowed_urls=None):
         """Initialize the queue state and task counters."""
         # Note: don't put a maximum size on the queue since it would
         # lead to deadlocks when all worker threads called put().
@@ -54,10 +54,14 @@ class UrlQueue (object):
         self.all_tasks_done = threading.Condition(self.mutex)
         self.unfinished_tasks = 0
         self.finished_tasks = 0
-        self.in_progress = {}
-        self.checked = LFUCache(size=10000)
+        self.in_progress = 0
         self.shutdown = False
-        self.unsorted = 0
+        # Each put() decreases the number of allowed puts.
+        # This way we can restrict the number of URLs that are checked.
+        if max_allowed_urls is not None and max_allowed_urls <= 0:
+            raise ValueError("Non-positive number of allowed URLs: %d" % max_allowed_urls)
+        self.max_allowed_urls = max_allowed_urls
+        self.num_puts = 0
 
     def qsize (self):
         """Return the approximate size of the queue (not reliable!)."""
@@ -65,22 +69,27 @@ class UrlQueue (object):
             return len(self.queue)
 
     def empty (self):
-        """Return True if the queue is empty, False otherwise (not reliable!)."""
+        """Return True if the queue is empty, False otherwise.
+        Result is thread-safe, but not reliable since the queue could have
+        been changed before the result is returned!"""
         with self.mutex:
             return self._empty()
 
     def _empty (self):
+        """Return True if the queue is empty, False otherwise.
+        Not thread-safe!"""
         return not self.queue
 
     def get (self, timeout=None):
         """Get first not-in-progress url from the queue and
-        return it. If no such url is available return None. The
-        url might be already cached.
+        return it. If no such url is available return None.
         """
         with self.not_empty:
             return self._get(timeout)
 
     def _get (self, timeout):
+        """Non thread-safe utility function of self.get() doing the real
+        work."""
         if timeout is None:
             while self._empty():
                 self.not_empty.wait()
@@ -93,33 +102,12 @@ class UrlQueue (object):
                 if remaining <= 0.0:
                     raise Empty()
                 self.not_empty.wait(remaining)
-        url_data = self.queue.popleft()
-        key = url_data.cache_url_key
-        if url_data.has_result:
-            # Already checked and copied from cache.
-            pass
-        elif key in self.checked:
-            # Already checked; copy result. And even ignore
-            # the case where url happens to be in_progress.
-            url_data.copy_from_cache(self.checked[key])
-        elif key in self.in_progress:
-            # It's being checked currently; put it back in the queue.
-            self.queue.append(url_data)
-            url_data = None
-        else:
-            self.in_progress[key] = url_data
-        return url_data
+        self.in_progress += 1
+        return self.queue.popleft()
 
-    def put (self, item, block=True, timeout=None):
+    def put (self, item):
         """Put an item into the queue.
-
-        If optional args 'block' is true and 'timeout' is None (the default),
-        block if necessary until a free slot is available. If 'timeout' is
-        a positive number, it blocks at most 'timeout' seconds and raises
-        the Full exception if no free slot was available within that time.
-        Otherwise ('block' is false), put an item on the queue if a free slot
-        is immediately available, else raise the Full exception ('timeout'
-        is ignored in that case).
+        Block if necessary until a free slot is available.
         """
         with self.mutex:
             self._put(item)
@@ -127,26 +115,43 @@ class UrlQueue (object):
 
     def _put (self, url_data):
         """Put URL in queue, increase number of unfished tasks."""
-        if self.shutdown:
-            # don't accept more URLs
+        if self.shutdown or self.max_allowed_urls == 0:
             return
-        log.debug(LOG_CACHE, "queueing %s", url_data)
-        key = url_data.cache_url_key
-        if key in self.checked:
-            # Put at beginning of queue to get consumed quickly.
-            url_data.copy_from_cache(self.checked[key])
-            self.queue.appendleft(url_data)
-        elif key in self.in_progress:
-            # Put at beginning of queue since it will be cached soon.
+        log.debug(LOG_CACHE, "queueing %s", url_data.url)
+        key = url_data.cache_url
+        cache = url_data.aggregate.result_cache
+        if url_data.has_result or cache.has_result(key):
             self.queue.appendleft(url_data)
         else:
+            assert key is not None, "no result for None key: %s" % url_data
+            if self.max_allowed_urls is not None:
+                self.max_allowed_urls -= 1
+            self.num_puts += 1
+            if self.num_puts >= NUM_PUTS_CLEANUP:
+                self.cleanup()
             self.queue.append(url_data)
-            self.unsorted += 1
-        if self.unsorted > 2000:
-            self._sort()
-            self.unsorted = 0
         self.unfinished_tasks += 1
 
+    def cleanup(self):
+        """Move cached elements to top."""
+        self.num_puts = 0
+        cached = []
+        for i, url_data in enumerate(self.queue):
+            key = url_data.cache_url
+            cache = url_data.aggregate.result_cache
+            if cache.has_result(key):
+                cached.append(i)
+        for pos in cached:
+            self._move_to_top(pos)
+
+    def _move_to_top(self, pos):
+        """Move element at given position to top of queue."""
+        if pos > 0:
+            self.queue.rotate(-pos)
+            item = self.queue.popleft()
+            self.queue.rotate(pos)
+            self.queue.appendleft(item)
+
     def task_done (self, url_data):
         """
         Indicate that a formerly enqueued task is complete.
@@ -163,54 +168,14 @@ class UrlQueue (object):
         placed in the queue.
         """
         with self.all_tasks_done:
-            log.debug(LOG_CACHE, "task_done %s", url_data)
-            if url_data is not None:
-                key = url_data.cache_url_key
-                if key is not None and key not in self.checked:
-                    self._cache_url(key, url_data)
-                else:
-                    assert key not in self.in_progress
+            log.debug(LOG_CACHE, "task_done %s", url_data.url)
             self.finished_tasks += 1
-            unfinished = self.unfinished_tasks - 1
-            if unfinished <= 0:
-                if unfinished < 0:
+            self.unfinished_tasks -= 1
+            self.in_progress -= 1
+            if self.unfinished_tasks <= 0:
+                if self.unfinished_tasks < 0:
                     raise ValueError('task_done() called too many times')
                 self.all_tasks_done.notifyAll()
-            self.unfinished_tasks = unfinished
-
-    def _cache_url (self, key, url_data):
-        """Put URL result data into cache."""
-        log.debug(LOG_CACHE, "Caching %r", key)
-        if key in self.in_progress:
-            del self.in_progress[key]
-        data = url_data.get_cache_data()
-        self.checked[key] = data
-        # check for aliases (eg. through HTTP redirections)
-        if hasattr(url_data, "aliases"):
-            data = url_data.get_alias_cache_data()
-            for key in url_data.aliases:
-                if key in self.checked or key in self.in_progress:
-                    continue
-                log.debug(LOG_CACHE, "Caching alias %r", key)
-                self.checked[key] = data
-
-    def _sort (self):
-        """Sort URL queue by putting all cached URLs at the beginning."""
-        newqueue = collections.deque()
-        while self.queue:
-            url_data = self.queue.popleft()
-            key = url_data.cache_url_key
-            if url_data.has_result:
-                # Already checked and copied from cache.
-                newqueue.appendleft(url_data)
-            elif key in self.checked:
-                # Already checked; copy result. And even ignore
-                # the case where url happens to be in_progress.
-                url_data.copy_from_cache(self.checked[key])
-                newqueue.appendleft(url_data)
-            else:
-                newqueue.append(url_data)
-        self.queue = newqueue
 
     def join (self, timeout=None):
         """Blocks until all items in the Queue have been gotten and processed.
@@ -248,22 +213,6 @@ class UrlQueue (object):
             self.shutdown = True
 
     def status (self):
-        """
-        Get tuple (finished tasks, in progress, queue size).
-        """
-        with self.mutex:
-            return (self.finished_tasks,
-                    len(self.in_progress), len(self.queue))
-
-    def checked_redirect (self, redirect, url_data):
-        """
-        Check if redirect is already in cache. Used for URL redirections
-        to avoid double checking of already cached URLs.
-        If the redirect URL is found in the cache, the result data is
-        already copied.
-        """
-        with self.mutex:
-            if redirect in self.checked:
-                url_data.copy_from_cache(self.checked[redirect])
-                return True
-            return False
+        """Get tuple (finished tasks, in progress, queue size)."""
+        # no need to acquire self.mutex since the numbers are unreliable anyways.
+        return (self.finished_tasks, self.in_progress, len(self.queue))
