@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2000-2010 Bastian Kleineidam
+# Copyright (C) 2000-2014 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,15 +19,12 @@ Handle FTP links.
 """
 
 import ftplib
-import urllib
 from cStringIO import StringIO
 
-from .. import log, LOG_CHECK, LinkCheckerError, fileutil
+from .. import log, LOG_CHECK, LinkCheckerError, mimeutil
 from . import proxysupport, httpurl, internpaturl, get_index_html
 from .const import WARN_FTP_MISSING_SLASH
 
-DEFAULT_TIMEOUT_SECS = 300
-
 
 class FtpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
     """
@@ -71,58 +68,44 @@ class FtpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
         self.files = []
         return None
 
-    def get_user_password (self):
-        """
-        Get credentials to use for login.
-        """
-        if self.userinfo:
-            return urllib.splitpasswd(self.userinfo)
-        return super(FtpUrl, self).get_user_password()
-
     def login (self):
-        """
-        Log into ftp server and check the welcome message.
-        """
-        # ready to connect
-        _user, _password = self.get_user_password()
-        host = self.urlparts[1]
-        key = ("ftp", host, _user, _password)
-        conn = self.aggregate.connections.get(key)
-        if conn is not None and conn.sock is not None:
-            # reuse cached FTP connection
-            self.url_connection = conn
-            return
-        self.aggregate.connections.wait_for_host(host)
+        """Log into ftp server and check the welcome message."""
+        self.url_connection = ftplib.FTP(timeout=self.aggregate.config["timeout"])
+        if log.is_debug(LOG_CHECK):
+            self.url_connection.set_debuglevel(1)
         try:
-            self.url_connection = ftplib.FTP()
-            if log.is_debug(LOG_CHECK):
-                self.url_connection.set_debuglevel(1)
-            host, port = proxysupport.parse_host_port(host)
-            self.url_connection.connect(host, port)
+            self.url_connection.connect(self.host, self.port)
+            _user, _password = self.get_user_password()
             if _user is None:
                 self.url_connection.login()
             elif _password is None:
                 self.url_connection.login(_user)
             else:
                 self.url_connection.login(_user, _password)
-        except EOFError, msg:
+            info = self.url_connection.getwelcome()
+            if info:
+                # note that the info may change every time a user logs in,
+                # so don't add it to the url_data info.
+                log.debug(LOG_CHECK, "FTP info %s", info)
+                pass
+            else:
+                raise LinkCheckerError(_("Got no answer from FTP server"))
+        except EOFError as msg:
             raise LinkCheckerError(
-                  _("Remote host has closed connection: %(msg)s") % str(msg))
-        info = self.url_connection.getwelcome()
-        if info:
-            # note that the info may change every time a user logs in,
-            # so don't add it to the url_data info.
-            log.debug(LOG_CHECK, "FTP info %s", info)
-        else:
-            raise LinkCheckerError(_("Got no answer from FTP server"))
+                      _("Remote host has closed connection: %(msg)s") % str(msg))
 
     def negotiate_encoding (self):
         """Check if server can handle UTF-8 encoded filenames.
         See also RFC 2640."""
-        features = self.url_connection.sendcmd("FEAT")
-        log.debug(LOG_CHECK, "FTP features %s", features)
-        if " UTF-8" in features.splitlines():
-            self.filename_encoding = "utf-8"
+        try:
+            features = self.url_connection.sendcmd("FEAT")
+        except ftplib.error_perm as msg:
+            log.debug(LOG_CHECK, "Ignoring error when getting FTP features: %s" % msg)
+            pass
+        else:
+            log.debug(LOG_CHECK, "FTP features %s", features)
+            if " UTF-8" in features.splitlines():
+                self.filename_encoding = "utf-8"
 
     def cwd (self):
         """
@@ -178,35 +161,25 @@ class FtpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
         self.url_connection.dir(add_entry)
         return files
 
-    def is_html (self):
-        """See if URL target is a HTML file by looking at the extension."""
-        mime = fileutil.guess_mimetype(self.url)
-        return self.ContentMimetypes.get(mime) == "html"
-
-    def is_css (self):
-        """See if URL target is a CSS file by looking at the extension."""
-        mime = fileutil.guess_mimetype(self.url)
-        return self.ContentMimetypes.get(mime) == "css"
-
     def is_parseable (self):
         """See if URL target is parseable for recursion."""
         if self.is_directory():
             return True
-        mime = fileutil.guess_mimetype(self.url, read=self.get_content)
-        return mime in self.ContentMimetypes
+        if self.content_type in self.ContentMimetypes:
+            return True
+        log.debug(LOG_CHECK, "URL with content type %r is not parseable.", self.content_type)
+        return False
 
     def is_directory (self):
         """See if URL target is a directory."""
-        return self.url.endswith('/')
+        # either the path is empty, or ends with a slash
+        path = self.urlparts[2]
+        return (not path) or path.endswith('/')
 
-    def parse_url (self):
-        """Parse URL target for links."""
-        if self.is_directory():
-            self.parse_html()
-            return
-        mime = fileutil.guess_mimetype(self.url, read=self.get_content)
-        key = self.ContentMimetypes[mime]
-        getattr(self, "parse_"+key)()
+    def set_content_type (self):
+        """Set URL content type, or an empty string if content
+        type could not be found."""
+        self.content_type = mimeutil.guess_mimetype(self.url, read=self.get_content)
 
     def read_content (self):
         """Return URL target content, or in case of directories a dummy HTML
@@ -214,29 +187,28 @@ class FtpUrl (internpaturl.InternPatternUrl, proxysupport.ProxySupport):
         if self.is_directory():
             self.url_connection.cwd(self.filename)
             self.files = self.get_files()
+            # XXX limit number of files?
             data = get_index_html(self.files)
         else:
             # download file in BINARY mode
             ftpcmd = "RETR %s" % self.filename
-            # XXX limit the download size to some sane value
             buf = StringIO()
             def stor_data (s):
                 """Helper method storing given data"""
+                # limit the download size
+                if (buf.tell() + len(s)) > self.max_size:
+                    raise LinkCheckerError(_("FTP file size too large"))
                 buf.write(s)
             self.url_connection.retrbinary(ftpcmd, stor_data)
             data = buf.getvalue()
             buf.close()
-        return data, len(data)
+        return data
 
     def close_connection (self):
-        """
-        Add the open connection to the connection pool.
-        """
-        if self.url_connection is None:
-            return
-        # add to cached connections
-        _user, _password = self.get_user_password()
-        key = ("ftp", self.urlparts[1], _user, _password)
-        cache_add = self.aggregate.connections.add
-        cache_add(key, self.url_connection, DEFAULT_TIMEOUT_SECS)
-        self.url_connection = None
+        """Release the open connection from the connection pool."""
+        if self.url_connection is not None:
+            try:
+                self.url_connection.quit()
+            except Exception:
+                pass
+            self.url_connection = None
