@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2005-2009 Bastian Kleineidam
+# Copyright (C) 2005-2014 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,15 +20,25 @@ Locking utility class.
 import threading
 from . import log, LOG_THREAD
 
-def get_lock (name):
-    return threading.Lock()
+def get_lock (name, debug=False):
+    """Get a new lock.
+    @param debug: if True, acquire() and release() will have debug messages
+    @ptype debug: boolean, default is False
+    @return: a lock object
+    @rtype: threading.Lock or DebugLock
+    """
+    lock = threading.Lock()
     # for thread debugging, use the DebugLock wrapper
-    #return DebugLock(threading.Lock(), name)
+    if debug:
+        lock = DebugLock(lock, name)
+    return lock
 
 
 class DebugLock (object):
     """Debugging lock class."""
+
     def __init__ (self, lock, name):
+        """Store lock and name parameters."""
         self.lock = lock
         self.name = name
 
@@ -44,3 +54,21 @@ class DebugLock (object):
         threadname = threading.currentThread().getName()
         log.debug(LOG_THREAD, "Release %s for %s", self.name, threadname)
         self.lock.release()
+
+
+def get_semaphore(name, value=None, debug=False):
+    """Get a new semaphore.
+    @param value: if not None, a BoundedSemaphore will be used
+    @ptype debug: int or None
+    @param debug: if True, acquire() and release() will have debug messages
+    @ptype debug: boolean, default is False
+    @return: a semaphore object
+    @rtype: threading.Semaphore or threading.BoundedSemaphore or DebugLock
+    """
+    if value is None:
+        lock = threading.Semaphore()
+    else:
+        lock = threading.BoundedSemaphore(value)
+    if debug:
+        lock = DebugLock(lock, name)
+    return lock
