@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2004-2010 Bastian Kleineidam
+# Copyright (C) 2004-2012 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,43 +17,45 @@
 """
 Test telnet checking.
 """
-from . import LinkCheckTest
+from .telnetserver import TelnetServerTest
 
 
-class TestTelnet (LinkCheckTest):
+class TestTelnet (TelnetServerTest):
     """Test telnet: link checking."""
 
-    def test_telnet (self):
+    def test_telnet_error (self):
         url = u"telnet:"
         nurl = self.norm(url)
         resultlines = [
             u"url %s" % url,
-            u"cache key %s" % nurl,
+            u"cache key None",
             u"real url %s" % nurl,
             u"error",
         ]
         self.direct(url, resultlines)
-        url = u"telnet://www.example.com"
+
+    def test_telnet_localhost (self):
+        url = self.get_url()
         resultlines = [
             u"url %s" % url,
             u"cache key %s" % url,
             u"real url %s" % url,
-            u"error",
+            u"valid",
         ]
         self.direct(url, resultlines)
-        url = u"telnet://user@www.example.com"
+        url = self.get_url(user=u"test")
         resultlines = [
             u"url %s" % url,
             u"cache key %s" % url,
             u"real url %s" % url,
-            u"error",
+            u"valid",
         ]
         self.direct(url, resultlines)
-        url = u"telnet://user:pass@www.example.com"
+        url = self.get_url(user=u"test", password=u"test")
         resultlines = [
             u"url %s" % url,
             u"cache key %s" % url,
             u"real url %s" % url,
-            u"error",
+            u"valid",
         ]
         self.direct(url, resultlines)
