@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2005-2010 Bastian Kleineidam
+# Copyright (C) 2005-2014 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -30,7 +30,7 @@ class TestLinkparser (unittest.TestCase):
 
     def _test_one_link (self, content, url):
         self.count_url = 0
-        h = linkparse.LinkFinder(content, self._test_one_url(url))
+        h = linkparse.LinkFinder(self._test_one_url(url), linkparse.LinkTags)
         p = linkcheck.HtmlParser.htmlsax.parser(h)
         h.parser = p
         try:
@@ -40,15 +40,29 @@ class TestLinkparser (unittest.TestCase):
             pass
         h.parser = None
         p.handler = None
+        self.assertEqual(self.count_url, 1)
 
     def _test_one_url (self, origurl):
         """Return parser callback function."""
         def callback (url, line, column, name, base):
             self.count_url += 1
-            self.assertEqual(self.count_url, 1)
             self.assertEqual(origurl, url)
         return callback
 
+    def _test_no_link (self, content):
+        def callback (url, line, column, name, base):
+            self.assertTrue(False, 'URL %r found' % url)
+        h = linkparse.LinkFinder(callback, linkparse.LinkTags)
+        p = linkcheck.HtmlParser.htmlsax.parser(h)
+        h.parser = p
+        try:
+            p.feed(content)
+            p.flush()
+        except linkparse.StopParse:
+            pass
+        h.parser = None
+        p.handler = None
+
     def test_href_parsing (self):
         # Test <a href> parsing.
         content = u'<a href="%s">'
@@ -61,6 +75,25 @@ class TestLinkparser (unittest.TestCase):
         url = u" alink "
         self._test_one_link(content % url, url)
 
+    def test_img_srcset_parsing(self):
+        content = u'<img srcset="%s 1x">'
+        url = u"imagesmall.jpg"
+        self._test_one_link(content % url, url)
+
+    def test_itemtype_parsing(self):
+        content = u'<div itemtype="%s">'
+        url = u"http://example.org/Movie"
+        self._test_one_link(content % url, url)
+
+    def test_form_parsing(self):
+        # Test <form action> parsing
+        content = u'<form action="%s">'
+        url = u"alink"
+        self._test_one_link(content % url, url)
+        content = u'<form action="%s" method="POST">'
+        url = u"alink"
+        self._test_no_link(content % url)
+
     def test_css_parsing (self):
         # Test css style attribute parsing.
         content = u'<table style="background: url(%s) no-repeat" >'
@@ -91,3 +124,8 @@ class TestLinkparser (unittest.TestCase):
         self.assertEqual(strip(content), "")
         content = "a/* */b/* */c"
         self.assertEqual(strip(content), "abc")
+
+    def test_url_quoting(self):
+        url = u'http://example.com/bla/a=b'
+        content = u'<a href="%s&quot;">'
+        self._test_one_link(content % url, url + u'"')
