@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2004-2010 Bastian Kleineidam
+# Copyright (C) 2004-2012 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -83,6 +83,8 @@ parsetests = [
     ("""<!- blubb ->""", """<!-- blubb -->"""),
     ("""<! -- blubb -->""", """<!-- blubb -->"""),
     ("""<!-- blubb -- >""", """<!-- blubb -->"""),
+    ("""<! blubb !>< a>""", """<!--blubb !--><a>"""),
+    ("""<! blubb >< a>""", """<!--blubb --><a>"""),
     # end tags
     ("""</a>""", """</a>"""),
     ("""</ a>""", """</a>"""),
@@ -163,11 +165,22 @@ parsetests = [
     ("""</td<td  a="b" >""", """</td><td a="b">"""),
     # missing beginning quote
     ("""<td a=b">""", """<td a="b">"""),
+    # stray < before start tag
+    ("""<0.<td  a="b" >""", """<0.<td a="b">"""),
+    # stray < before end tag
+    ("""<0.</td >""", """<0.</td>"""),
     # missing end quote (XXX TODO)
     #("""<td a="b>\n""", """<td a="b">\n"""),
     #("""<td a="b></td>\na""", """<td a="b"></td>\na"""),
     #("""<a  b="c><a b="c>\n""", """<a b="c"><a b="c">\n"""),
     #("""<td a="b c="d"></td>\n""", """<td a="b" c="d"></td>\n"""),
+    # HTML5 tags
+    ("""<audio  src=bla>""", """<audio src="bla">"""),
+    ("""<button  formaction=bla>""", """<button formaction="bla">"""),
+    ("""<html  manifest=bla>""", """<html manifest="bla">"""),
+    ("""<source  src=bla>""", """<source src="bla">"""),
+    ("""<track  src=bla>""", """<track src="bla">"""),
+    ("""<video  src=bla>""", """<video src="bla">"""),
 ]
 
 flushtests = [
@@ -191,9 +204,7 @@ class TestParser (unittest.TestCase):
         self.htmlparser2 = linkcheck.HtmlParser.htmlsax.parser()
 
     def test_parse (self):
-        """
-        Parse all test patterns in one go.
-        """
+        # Parse all test patterns in one go.
         for _in, _out in parsetests:
             out = StringIO()
             handler = linkcheck.HtmlParser.htmllib.HtmlPrettyPrinter(out)
@@ -213,9 +224,7 @@ class TestParser (unittest.TestCase):
         htmlparser.reset()
 
     def test_feed (self):
-        """
-        Parse all test patterns sequentially.
-        """
+        # Parse all test patterns sequentially.
         for _in, _out in parsetests:
             out = StringIO()
             handler = linkcheck.HtmlParser.htmllib.HtmlPrettyPrinter(out)
@@ -225,9 +234,7 @@ class TestParser (unittest.TestCase):
             self.check_results(self.htmlparser, _in, _out, out)
 
     def test_interwoven (self):
-        """
-        Parse all test patterns on two parsers interwoven.
-        """
+        # Parse all test patterns on two parsers interwoven.
         for _in, _out in parsetests:
             out = StringIO()
             out2 = StringIO()
@@ -255,9 +262,7 @@ class TestParser (unittest.TestCase):
             self.assertEqual(out.getvalue(), out2.getvalue())
 
     def test_flush (self):
-        """
-        Test parser flushing.
-        """
+        # Test parser flushing.
         for _in, _out in flushtests:
             out = StringIO()
             handler = linkcheck.HtmlParser.htmllib.HtmlPrettyPrinter(out)
@@ -266,10 +271,39 @@ class TestParser (unittest.TestCase):
             self.check_results(self.htmlparser, _in, _out, out)
 
     def test_entities (self):
-        """
-        Test entity resolving.
-        """
+        # Test entity resolving.
         resolve = linkcheck.HtmlParser.resolve_entities
         for c in "abcdefghijklmnopqrstuvwxyz":
             self.assertEqual(resolve("&#%d;" % ord(c)), c)
         self.assertEqual(resolve("&#1114112;"), u"")
+
+    def test_peek (self):
+        # Test peek() parser function
+        data = '<a href="test.html">name</a>'
+
+        class NamePeeker (object):
+
+            def start_element (self_handler, tag, attrs):
+                # use self reference of TestParser instance
+                self.assertRaises(TypeError, self.htmlparser.peek, -1)
+                self.assertEqual(self.htmlparser.peek(0), "")
+                self.assertEqual(self.htmlparser.peek(4), "name")
+
+        self.htmlparser.handler = NamePeeker()
+        self.htmlparser.feed(data)
+
+    def test_encoding_detection (self):
+        html = '<meta http-equiv="content-type" content="text/html; charset=UTF-8">'
+        self.encoding_test(html, "utf-8")
+        html = '<meta charset="UTF-8">'
+        self.encoding_test(html, "utf-8")
+        html = '<meta charset="hulla">'
+        self.encoding_test(html, "iso8859-1")
+        html = '<meta http-equiv="content-type" content="text/html; charset=blabla">'
+        self.encoding_test(html, "iso8859-1")
+
+    def encoding_test (self, html, expected):
+        parser = linkcheck.HtmlParser.htmlsax.parser()
+        self.assertEqual(parser.encoding, "iso8859-1")
+        parser.feed(html)
+        self.assertEqual(parser.encoding, expected)
