@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2006-2010 Bastian Kleineidam
+# Copyright (C) 2006-2014 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -166,8 +166,8 @@ class TestRobotsTxt (unittest.TestCase):
         ]
         self.rp.parse(lines)
         self.assertEqual(str(self.rp), "\n".join(lines2))
-        good = ['/','/test.html']
-        bad = ['/cyberworld/map/index.html','/tmp/xxx','/foo.html']
+        good = ['/', '/test.html']
+        bad = ['/cyberworld/map/index.html', '/tmp/xxx', '/foo.html']
         self.check_urls(good, bad)
 
     def test_access2 (self):
@@ -191,7 +191,7 @@ class TestRobotsTxt (unittest.TestCase):
         ]
         self.rp.parse(lines)
         self.assertEqual(str(self.rp), "\n".join(lines2))
-        good = ['/','/test.html',('cybermapper','/cyberworld/map/index.html')]
+        good = ['/', '/test.html', ('cybermapper', '/cyberworld/map/index.html')]
         bad = ['/cyberworld/map/index.html']
         self.check_urls(good, bad)
 
@@ -208,7 +208,7 @@ class TestRobotsTxt (unittest.TestCase):
         self.rp.parse(lines)
         self.assertEqual(str(self.rp), "\n".join(lines2))
         good = []
-        bad = ['/cyberworld/map/index.html','/','/tmp/']
+        bad = ['/cyberworld/map/index.html', '/', '/tmp/']
         self.check_urls(good, bad)
 
     def test_access4 (self):
@@ -229,8 +229,8 @@ class TestRobotsTxt (unittest.TestCase):
         self.rp.parse(lines)
         self.assertEqual(str(self.rp), "\n".join(lines2))
         good = []
-        bad = ['/tmp','/tmp.html','/tmp/a.html',
-               '/a%3cd.html','/a%3Cd.html','/a%2fb.html',
+        bad = ['/tmp', '/tmp.html', '/tmp/a.html',
+               '/a%3cd.html', '/a%3Cd.html', '/a%2fb.html',
                '/~joe/index.html', '/a/b.html',
         ]
         self.check_urls(good, bad, 'figtree')
@@ -253,9 +253,9 @@ class TestRobotsTxt (unittest.TestCase):
         ]
         self.rp.parse(lines)
         self.assertEqual(str(self.rp), "\n".join(lines2))
-        good = ['/tmp',] # XFAIL: '/a%2fb.html'
-        bad = ['/tmp/','/tmp/a.html',
-               '/a%3cd.html','/a%3Cd.html',"/a/b.html",
+        good = ['/tmp'] # XFAIL: '/a%2fb.html'
+        bad = ['/tmp/', '/tmp/a.html',
+               '/a%3cd.html', '/a%3Cd.html', "/a/b.html",
                '/%7Ejoe/index.html']
         self.check_urls(good, bad)
 
@@ -271,7 +271,7 @@ class TestRobotsTxt (unittest.TestCase):
 
     def test_access7 (self):
         lines = [
-            "User-agent: Example*",
+            "User-agent: Example",
             "Disallow: /example",
             "",
             "User-agent: *",
@@ -291,3 +291,10 @@ class TestRobotsTxt (unittest.TestCase):
         self.check_url("spam", "/cgi-bin?a=1", False)
         self.check_url("spam", "/", True)
 
+    def test_sitemap(self):
+        lines = [
+            "Sitemap: bla",
+        ]
+        self.rp.parse(lines)
+        self.assertTrue(len(self.rp.sitemap_urls) > 0)
+        self.assertTrue(self.rp.sitemap_urls[0] == ("bla", 1))
