@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2004-2010 Bastian Kleineidam
+# Copyright (C) 2004-2014 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,6 +20,7 @@ Test string formatting operations.
 
 import unittest
 import os
+import time
 import linkcheck.strformat
 
 
@@ -29,9 +30,7 @@ class TestStrFormat (unittest.TestCase):
     """
 
     def test_unquote (self):
-        """
-        Test quote stripping.
-        """
+        # Test quote stripping.
         u = linkcheck.strformat.unquote
         self.assertEqual(u(""), "")
         self.assertEqual(u(None), None)
@@ -54,16 +53,14 @@ class TestStrFormat (unittest.TestCase):
         self.assertEqual(u("\"a'", matching=True), "\"a'")
 
     def test_wrap (self):
-        """
-        Test line wrapping.
-        """
+        # Test line wrapping.
         wrap = linkcheck.strformat.wrap
         s = "11%(sep)s22%(sep)s33%(sep)s44%(sep)s55" % {'sep': os.linesep}
         # testing width <= 0
         self.assertEqual(wrap(s, -1), s)
         self.assertEqual(wrap(s, 0), s)
         l = len(os.linesep)
-        gap = " "*l
+        gap = " "
         s2 = "11%(gap)s22%(sep)s33%(gap)s44%(sep)s55" % \
              {'sep': os.linesep, 'gap': gap}
         # splitting lines
@@ -76,25 +73,20 @@ class TestStrFormat (unittest.TestCase):
         self.assertEqual(wrap(None, 10), None)
         self.assertFalse(linkcheck.strformat.get_paragraphs(None))
 
-
     def test_remove_markup (self):
-        """
-        Test markup removing.
-        """
+        # Test markup removing.
         self.assertEqual(linkcheck.strformat.remove_markup("<a>"), "")
         self.assertEqual(linkcheck.strformat.remove_markup("<>"), "")
         self.assertEqual(linkcheck.strformat.remove_markup("<<>"), "")
         self.assertEqual(linkcheck.strformat.remove_markup("a < b"), "a < b")
 
     def test_strsize (self):
-        """
-        Test byte size strings.
-        """
+        # Test byte size strings.
         self.assertRaises(ValueError, linkcheck.strformat.strsize, -1)
         self.assertEqual(linkcheck.strformat.strsize(0), "0B")
         self.assertEqual(linkcheck.strformat.strsize(1), "1B")
         self.assertEqual(linkcheck.strformat.strsize(2), "2B")
-        self.assertEqual(linkcheck.strformat.strsize(1023), "1023B")
+        self.assertEqual(linkcheck.strformat.strsize(1023, grouping=False), "1023B")
         self.assertEqual(linkcheck.strformat.strsize(1024), "1KB")
         self.assertEqual(linkcheck.strformat.strsize(1024*25), "25.00KB")
         self.assertEqual(linkcheck.strformat.strsize(1024*1024), "1.00MB")
@@ -114,11 +106,12 @@ class TestStrFormat (unittest.TestCase):
         self.assertEqual(linkcheck.strformat.indent(s, ""), s)
         self.assertEqual(linkcheck.strformat.indent(s, " "), " "+s)
 
-    def test_stripall (self):
-        self.assertEqual(linkcheck.strformat.stripall("a\tb"), "ab")
-        self.assertEqual(linkcheck.strformat.stripall(" a\t b"), "ab")
-        self.assertEqual(linkcheck.strformat.stripall(" \r\na\t \nb\r"), "ab")
-        self.assertEqual(linkcheck.strformat.stripall(None), None)
+    def test_stripurl (self):
+        self.assertEqual(linkcheck.strformat.stripurl(u"a\tb"), u"a\tb")
+        self.assertEqual(linkcheck.strformat.stripurl(u" a\t b"), u"a\t b")
+        self.assertEqual(linkcheck.strformat.stripurl(u" ab\t\ra\nb"), u"ab")
+        self.assertEqual(linkcheck.strformat.stripurl(None), None)
+        self.assertEqual(linkcheck.strformat.stripurl(u""), u"")
 
     def test_limit (self):
         self.assertEqual(linkcheck.strformat.limit("", 0), "")
@@ -128,8 +121,8 @@ class TestStrFormat (unittest.TestCase):
 
     def test_strtime (self):
         zone = linkcheck.strformat.strtimezone()
-        t = linkcheck.strformat.strtime(0)
-        self.assertEqual(t, "1970-01-01 01:00:00"+zone)
+        t = linkcheck.strformat.strtime(0, func=time.gmtime)
+        self.assertEqual(t, "1970-01-01 00:00:00"+zone)
 
     def test_duration (self):
         duration = linkcheck.strformat.strduration
@@ -144,7 +137,7 @@ class TestStrFormat (unittest.TestCase):
         self.assertEqual(duration(60*60*24), "24:00:00")
 
     def test_duration_long (self):
-        duration = linkcheck.strformat.strduration_long
+        duration = lambda s: linkcheck.strformat.strduration_long(s, do_translate=False)
         self.assertEqual(duration(-0.5), "-0.50 seconds")
         self.assertEqual(duration(0), "0.00 seconds")
         self.assertEqual(duration(0.9), "0.90 seconds")
@@ -179,3 +172,14 @@ class TestStrFormat (unittest.TestCase):
         self.assertEqual(ascii_safe("a"), "a")
         self.assertEqual(ascii_safe(u"a"), "a")
         self.assertEqual(ascii_safe(u""), "")
+
+    def test_strip_control_chars(self):
+        strip = linkcheck.strformat.strip_control_chars
+        self.assertEqual(strip(""), "")
+        self.assertEqual(strip(u""), u"")
+        self.assertEqual(strip("a"), "a")
+        self.assertEqual(strip(u"a"), u"a")
+        self.assertEqual(strip(""), "")
+        self.assertEqual(strip(u""), u"")
+        self.assertEqual(strip("\x01"), "")
+        self.assertEqual(strip(u"\x01"), u"")
