@@ -1,5 +1,5 @@
 # -*- coding: iso-8859-1 -*-
-# Copyright (C) 2004-2010 Bastian Kleineidam
+# Copyright (C) 2004-2014 Bastian Kleineidam
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,7 +21,9 @@ Test url build method from url data objects.
 import unittest
 import linkcheck.configuration
 import linkcheck.director
-import linkcheck.checker.httpurl
+import linkcheck.checker.urlbase
+from linkcheck.checker import get_url_from
+
 
 def get_test_aggregate ():
     """
@@ -39,10 +41,34 @@ class TestUrlBuild (unittest.TestCase):
 
     def test_http_build (self):
         parent_url = "http://localhost:8001/tests/checker/data/http.html"
-        base_url = "http://"
+        base_url = "http://foo"
         recursion_level = 0
         aggregate = get_test_aggregate()
-        o = linkcheck.checker.httpurl.HttpUrl(base_url, recursion_level,
+        o = get_url_from(base_url, recursion_level, aggregate, parent_url=parent_url)
+        o.build_url()
+        self.assertEqual(o.url, u'http://foo')
+
+    def test_urljoin (self):
+        parent_url = "http://localhost:8001/test"
+        base_url = ";param=value"
+        res = linkcheck.checker.urlbase.urljoin(parent_url, base_url)
+        self.assertEqual(res, 'http://localhost:8001/;param=value')
+
+    def test_urljoin_file (self):
+        parent_url = "file:///a/b.html"
+        base_url = "?c=d"
+        recursion_level = 0
+        aggregate = get_test_aggregate()
+        o = get_url_from(base_url, recursion_level,
                aggregate, parent_url=parent_url)
         o.build_url()
-        self.assertEqual(o.url, 'http://')
+        self.assertEqual(o.url, parent_url)
+
+    def test_http_build2 (self):
+        parent_url = u'http://example.org/test?a=b&c=d'
+        base_url = u'#usemap'
+        recursion_level = 0
+        aggregate = get_test_aggregate()
+        o = get_url_from(base_url, recursion_level, aggregate, parent_url=parent_url)
+        o.build_url()
+        self.assertEqual(o.url, parent_url+base_url)
